/*global base64js */
'use strict';

// Declare app level module which depends on views, and components
angular.module('myApp', ['MIDI', 'ui.knob', 'ui.bootstrap', 'rzModule', 'Patch',
	'ngRoute',
	'myApp.main',
	'myApp.version'
])

.config(['$routeProvider', function ($routeProvider) {
		$routeProvider.otherwise({redirectTo: '/main'});
	}])

.run(function () {

/*	navigator.permissions.query({name:'midi', sysex:true}).then(function(permissionStatus) {  
		console.log('MIDI perms: '+permissionStatus);
	});
*/
})

.controller('indexCtrl', ['$scope', '$uibModal', '$log', 'MIDIService', 'MIDIRequestService', 'PatchService',
function ($scope, $uibModal, $log, MIDIService, MIDIRequestService, PatchService) {

	$scope.sendAllNotesOff = function () {
		MIDIService.sendAllNotesOff();
	};

	$scope.getCurrentBuffer = function () {
		$scope.soundName = undefined;
		MIDIRequestService.requestEditBuffer(function (aData) {
			PatchService.initPatch(aData);
		});
	};
	
	$scope.sendToEditBuffer = function() {
		var oData = PatchService.getDataToSendToEditBuffer();
		MIDIService.sendData(oData);
	};

	$scope.midiDevices = function () {
		var modalInstance = $uibModal.open({
			templateUrl: 'midiDevicesModal.html',
			controller: 'MidiDevicesCtrl'
		});
	};
	
	function download(filename, dataBase64String) {
		var pom = document.createElement('a');
		pom.setAttribute('href', 'data:application/octet-stream;base64,' + dataBase64String);
		pom.setAttribute('download', filename);

		if (document.createEvent) {
			var event = document.createEvent('MouseEvents');
			event.initEvent('click', true, true);
			pom.dispatchEvent(event);
		}
		else {
			pom.click();
		}
	}
	
	$scope.download = function() {
		var modalInstance = $uibModal.open({
			templateUrl: 'downloadModal.html',
			controller: 'DownloadCtrl'
		});

		modalInstance.result.then(function (aFilename) {						
			var oData = PatchService.getData();
			var oBase64 = base64js.fromByteArray(oData);
			download(aFilename, oBase64);
		});
	};
	
	
	$scope.upload = function() {
		var modalInstance = $uibModal.open({
			templateUrl: 'uploadModal.html',
			controller: 'UploadCtrl'
		});

		modalInstance.result.then(function (aFilename) {
			$scope.soundName = aFilename;
		});
	};
	
	
}])

.controller('MidiDevicesCtrl', ['$scope', '$uibModalInstance', '$timeout', 'MIDIService', 
function ($scope, $uibModalInstance, $timeout, MIDIService) {

	$scope.inputs = MIDIService.getInputs();
	$scope.outputs = MIDIService.getOutputs();
	$scope.midiInput = MIDIService.getInputId();
	$scope.midiOutput = MIDIService.getOutputId();
	$scope.debug = MIDIService.getDebugEnabled();
	
	$scope.throttle = MIDIService.getThrottleDelay();
	$scope.throttle_options = {
		floor: 0, 
		ceil: 100, 
		value: $scope.throttle,
		vertical: false,
		getPointerColor: function() { return '#245269'; },
		getSelectionBarColor: function() { return '#777777'; }
	};

	// https://github.com/angular-slider/angularjs-slider/issues/79
	$timeout(function () {
		$scope.$broadcast('rzSliderForceRender');
	});

	$scope.ok = function () {
		$uibModalInstance.close();
		
		MIDIService.selectInput($scope.midiInput);
		MIDIService.selectOutput($scope.midiOutput);
		MIDIService.setDebugEnabled($scope.debug);
		MIDIService.setThrottleDelay($scope.throttle);
	};

	$scope.cancel = function () {
		$uibModalInstance.dismiss('cancel');
	};
}])

.controller('UploadCtrl', ['$scope', '$uibModalInstance', 'PatchService', 'MIDIService', 
function ($scope, $uibModalInstance, PatchService, MIDIService) {
	
	$scope.ok = function () {
		$scope.error = undefined;
		// c'est pas genial de faire le load ici mais je ne vois pas comment renvoyer le nom du fichier 
		// sans fermer le dialogue, que je dois maintenir ouvert en cas d'erreur.
		var reader = new FileReader();			
		reader.onloadend = function(e){
			//console.log(e.target.result);
			// e.target.result est un ArrayBuffer mais on ne peut pas le lire directement ce serait trop simple...
			var oDataView = new DataView(e.target.result);
			var oError = PatchService.initPatchFromDataView(oDataView);
			if (oError) {
				$scope.$apply(function() {
					$scope.error = oError;
				});
			}
			else {
				var oData = PatchService.getDataToSendToEditBuffer();
				MIDIService.sendData(oData);
				$uibModalInstance.close($scope.soundFile.name);
			}
		};
		reader.readAsArrayBuffer($scope.soundFile);
	};

	$scope.cancel = function () {
		$uibModalInstance.dismiss('cancel');
	};
}])

.controller('DownloadCtrl', ['$scope', '$uibModalInstance', function ($scope, $uibModalInstance) {
	$scope.filename = 'sound.m1000p';
	$scope.ok = function () {
		// TODO check name
		$uibModalInstance.close($scope.filename);
	};

	$scope.cancel = function () {
		$uibModalInstance.dismiss('cancel');
	};
}])

// http://uncorkedstudios.com/blog/multipartformdata-file-upload-with-angularjs
.directive('fileModel', ['$parse', function ($parse) {
	return {
		restrict: 'A',
		link: function(scope, element, attrs) {
			var model = $parse(attrs.fileModel);
			//console.log('model: '+model);
			var modelSetter = model.assign;

			element.bind('change', function(){
				scope.$apply(function() {
					//console.log('model set');
					modelSetter(scope, element[0].files[0]);
				});
			});
		}
	};
}])
;
