'use strict';

angular.module('myApp.main', ['ngRoute', 'MIDI', 'Patch'])

.config(['$routeProvider', function ($routeProvider) {
	$routeProvider.when('/main', {
		templateUrl: 'main/main.html',
		controller: 'MainCtrl'
	});
}])

.controller('MainCtrl', ['$scope', '$rootScope', '$log', 'MIDIService', 'MIDIRequestService', 'PatchService', 'uibButtonConfig',  
function ($scope, $rootScope, $log, MIDIService, MIDIRequestService, PatchService, uibButtonConfig) {

	var oMainColor = '#494B52';
	var oDefaultKnobOptions = {
		size: 60,
		startAngle: -150,
		endAngle: 150,
		skin: {
			type: 'tron',
			width: 1,
			color: oMainColor,
			spaceWidth: 1
		},
		subText: {
			enabled: true,
			text: 'Filter',
			font: 11,
			color: "black",
			offsetY: 0
		},
		barColor: oMainColor,
		textColor: oMainColor,
		textOffsetY: -10,
		trackWidth: 5,
		barWidth: 5,
		fontSize: 12,
		min: 0,
		max: 63,
		animate: {
			enabled: false
		}
	};

	var parameters = [];
	var signedParameters = [];
	var bitFieldParameters = [];
	var selectParameters = [];
	
	
	uibButtonConfig.activeClass = 'btn_active';
	
	function setupKnob(aParamIndex, aParamNumber, aName, aLabel, aMin, aMax, aSize) {
		var oVarOptions = aName+'_options';
		var oVarValue = aName+'_value';
		$scope[oVarOptions] = JSON.parse(JSON.stringify(oDefaultKnobOptions));
		$scope[oVarOptions].subText.text = aLabel;
		$scope[oVarOptions].min = aMin;
		$scope[oVarOptions].max = aMax;
		if (aParamIndex === -1) {
			$scope[oVarOptions].readOnly = true;
		}
		else {
			if (aSize) {
				$scope[oVarOptions].size = aSize;
			}
			$scope.$watch(oVarValue, function() {
				MIDIRequestService.sendParameter(aParamNumber, $scope[oVarValue]);
				PatchService.setValueAtIndex($scope[oVarValue], aParamIndex);
			});
			var oEntry = {
				knobValue:  oVarValue,
				paramIndex: aParamIndex
			};
			if (aMin < 0) {
				signedParameters.push(oEntry);
			}
			else {
				parameters.push(oEntry);
			}
		}
	}

	function setupMatrixKnob(aPath) {
		var oModel = $scope.matrix_parameters[aPath];
		oModel.options = JSON.parse(JSON.stringify(oDefaultKnobOptions));
		oModel.options.size = 45;
		oModel.options.subText.text = '';
		oModel.options.min = -63;
		oModel.options.max = 63;
		oModel.options.textOffsetY = 0;
		function changeHandler(newValue) {
			var oSource = $scope.matrix_parameters[aPath].source;
			var oDest = $scope.matrix_parameters[aPath].dest;
			var oValue = $scope.matrix_parameters[aPath].value;
			MIDIRequestService.sendMatrixParameter(aPath, oSource, oDest, oValue);
			PatchService.setValueAtIndex(oSource, 104+aPath*3);
			PatchService.setValueAtIndex(oValue, 105+aPath*3);
			PatchService.setValueAtIndex(oDest, 106+aPath*3);
		}
		$scope.$watch(function() {
			return oModel.value;
		}, changeHandler);
		$scope.$watch(function() {
			return oModel.source;
		}, changeHandler);
		$scope.$watch(function() {
			return oModel.dest;
		}, changeHandler);
		return oModel;
	}
		
	function setupFields(aParamIndex, aParamNumber, aName, aProperties) {
		var oVar = $scope[aName];
		$scope.$watchCollection(aName, function() {
			var oValue = 0;
			for (var i = 0; i < aProperties.length; i++) {
				var oBitVal = oVar[aProperties[i]] ? 1 : 0;				
				oValue &= ~(1 << i);
				oValue |= oBitVal << i;
			}
			MIDIRequestService.sendParameter(aParamNumber, oValue);
			PatchService.setValueAtIndex(oValue, aParamIndex);
		});
		bitFieldParameters.push({
			varName: aName,
			paramIndex: aParamIndex,
			properties: aProperties
		});
	}

	/**
	 * Création d'un select avec des valeurs textuelles.
	 * La valeur correspond à l'index choisi.
	 * @param {type} aParamIndex
	 * @param {type} aParamNumber
	 * @param {type} aName
	 * @param {Array} aValues tableau des valeurs string
	 * @returns {undefined}
	 */
	function setupSelect(aParamIndex, aParamNumber, aName, aValues) {
		$scope.$watch(aName, function() {
			var oSelectedValue = $scope[aName];
			for (var oValue = 0; oValue < aValues.length; oValue++) {
				if (oSelectedValue === aValues[oValue]) {
					break;
				}
			}
			MIDIRequestService.sendParameter(aParamNumber, oValue);
			PatchService.setValueAtIndex(oValue, aParamIndex);
		});
		selectParameters.push({
			name:  aName,
			paramIndex: aParamIndex,
			values: aValues
		});
	}

	$scope.modulation_sources = ["None", "Envelope 1", "Envelope 2", "Envelope 3", "LFO 1", "LFO 2", "Vibrato", "Ramp 1", "Ramp 2", 
		"Keyboard", "Portamento", "Tracking generator", "Keyboard gate", "Velocity", "Release velocity", 
		"Pressure", "Pedal 1", "Pedal 2", "Pitch bend", "Lever 2", "Lever 3"];
	$scope.modulation_destinations = ["None", "DCO1 Frequency", "DCO1 Pulse width", "DCO1 Waveshape",
		"DCO2 Frequency", "DCO2 Pulse width", "DCO2 Waveshape", "Mix level", "VCF FM Amount", "VCF Frequency", "VCF Resonance", "VCA1 Level", "VCA2 Level",
		"Envelope1 Delay", "Envelope1 Attack", "Envelope1 Decay", "Envelope1 Release", "Envelope1 Amplitude",
		"Envelope2 Delay", "Envelope2 Attack", "Envelope2 Decay", "Envelope2 Release", "Envelope2 Amplitude",
		"Envelope3 Delay", "Envelope3 Attack", "Envelope3 Decay", "Envelope3 Release", "Envelope3 Amplitude",
		"LFO1 Speed", "LFO1 Amplitude", "LFO2 Speed", "LFO2 Amplitude", "Portamento Time"];
		
	// DCO1
	setupKnob(10, 5, 'dco1_wave', 'WAVE', 0, 63);
	setupKnob(9, 0, 'dco1_freq', 'FREQ', 0, 63);
	setupKnob(86, 1, 'dco1_lfo1_freq', '<LFO1', -63, 63);
	setupKnob(11, 3, 'dco1_pw', 'PW', 0, 63);
	setupKnob(87, 4, 'dco1_lfo2_pw', '<LFO2', -63, 63);
	$scope.dco1_wave_select = { pulse: false, wave: false	};
	setupFields(13, 6, 'dco1_wave_select', ['pulse', 'wave']);
	$scope.dco1_fixmod = { pitchbend: false, vibrato: false };
	$scope.dco1_fixmod2 = { portamento: false };
	setupFields(12, 7, 'dco1_fixmod', ['pitchbend', 'vibrato']);
	setupFields(21, 8, 'dco1_fixmod2', ['portamento']);
	$scope.dco1_clic = {clic: false};
	setupFields(22, 9, 'dco1_clic', ['clic']);

	// DCO2
	setupKnob(15, 15, 'dco2_wave', 'WAVE', 0, 63);
	setupKnob(14, 10, 'dco2_freq', 'FREQ', 0, 63);
	setupKnob(88, 11, 'dco2_lfo1_freq', '<LFO1', -63, 63);
	setupKnob(16, 13, 'dco2_pw', 'PW', 0, 63);
	setupKnob(89, 14, 'dco2_lfo2_pw', '<LFO2', -63, 63);
	setupKnob(19, 12, 'dco2_detune', 'DETUNE', -31, 31);
	$scope.dco2_wave_select = { pulse: false, wave: false, noise: false };
	setupFields(18, 16, 'dco2_wave_select', ['pulse', 'wave', 'noise']);
	$scope.dco2_fixmod = {	pitchbend: false, vibrato: false };
	$scope.dco2_fixmod2 = { portamento: false, keyboard: false };
	setupFields(17, 17, 'dco2_fixmod', ['pitchbend', 'vibrato']);
	setupFields(23, 18, 'dco2_fixmod2', ['portamento', 'keyboard']);
	$scope.dco2_clic = {clic: false};
	setupFields(24, 19, 'dco2_clic', ['clic']);
	$scope.dco2_sync = 'none';
	setupSelect(25, 2, 'dco2_sync', ['none', 'soft', 'hard', 'harder']);
		
	// LFO1
	setupKnob(41, 84, 'lfo1_amp', 'AMP', 0, 63);
	setupKnob(97, 85, 'lfo1_amp_ramp1', '<RAMP1', -63, 63);
	setupKnob(35, 80, 'lfo1_speed', 'SPEED', 0, 63);
	setupKnob(102, 81, 'lfo1_speed_press', '<PRESS', -63, 63);
	setupKnob(39, 83, 'lfo1_retrig_point', 'RETRIG PT', 0, 63);
	$scope.lfo1_lag_enable = {lag: false};
	setupFields(37, 87, 'lfo1_lag_enable', ['lag']);
	$scope.lfo_triggers = ['None', 'Single trigger', 'Multi trigger', 'External trigger'];
	$scope.lfo_waves = ["Triangle", "Up sawtooth", "Down sawtooth", "Square", "Random", "Noise", "Sampled"];
	$scope.lfo1_trigger = 0;
	$scope.$watch('lfo1_trigger', function() {
		MIDIRequestService.sendParameter(86, $scope.lfo1_trigger);
		PatchService.setValueAtIndex($scope.lfo1_trigger, 36);
	});
	$scope.lfo1_wave = 0;
	$scope.$watch('lfo1_wave', function() {
		MIDIRequestService.sendParameter(82, $scope.lfo1_wave);
		PatchService.setValueAtIndex($scope.lfo1_wave, 38);
	});
	$scope.lfo1_sample_source = 0;
	$scope.$watch('lfo1_sample_source', function() {
		MIDIRequestService.sendParameter(88, $scope.lfo1_sample_source);
		PatchService.setValueAtIndex($scope.lfo1_sample_source, 40);
	});

	// LFO2
	setupKnob(48, 94, 'lfo2_amp', 'AMP', 0, 63);
	setupKnob(98, 95, 'lfo2_amp_ramp1', '<RAMP1', -63, 63);
	setupKnob(42, 90, 'lfo2_speed', 'SPEED', 0, 63);
	setupKnob(103, 91, 'lfo2_speed_press', '<PRESS', -63, 63);
	setupKnob(46, 93, 'lfo2_retrig_point', 'RETRIG PT', 0, 63);
	$scope.lfo2_lag_enable = {lag: false};
	setupFields(44, 97, 'lfo2_lag_enable', ['lag']);
	$scope.lfo2_trigger = 0;
	$scope.$watch('lfo2_trigger', function() {
		MIDIRequestService.sendParameter(96, $scope.lfo2_trigger);
		PatchService.setValueAtIndex($scope.lfo2_trigger, 43);
	});
	$scope.lfo2_wave = 0;
	$scope.$watch('lfo2_wave', function() {
		MIDIRequestService.sendParameter(92, $scope.lfo2_wave);
		PatchService.setValueAtIndex($scope.lfo2_wave, 45);
	});
	$scope.lfo2_sample_source = 0;
	$scope.$watch('lfo2_sample_source', function() {
		MIDIRequestService.sendParameter(98, $scope.lfo2_sample_source);
		PatchService.setValueAtIndex($scope.lfo2_sample_source, 47);
	});
		
	// common
	setupKnob(20, 20, 'mix', 'MIX', 0, 63);
	setupKnob(31, 27, 'vca1', 'VCA1', 0, 63);
	setupKnob(92, 28, 'vca1_vel', '<VEL', -63, 63);
	setupKnob(-1, -1, 'vca2', 'VCA2', 0, 63);
	setupKnob(93, 29, 'vca2_env2', '<ENV2', -63, 63);
	$scope.keyboard_mode = 'reassign';
	setupSelect(8, 48, 'keyboard_mode', ['reassign', 'rotate', 'unison', 'reassign_rob']);
	
	// Filter
	setupKnob(26, 21, 'filter', 'FREQ', 0, 127);
	setupKnob(27, 24, 'filter_res', 'RES', 0, 63);
	setupKnob(30, 30, 'filter_fm', 'FM', 0, 63);
	setupKnob(90, 22, 'filter_freq_env1', 'FR<ENV1', -63, 63);
	setupKnob(91, 23, 'filter_freq_press', 'FR<PRESS', -63, 63);
	setupKnob(100, 31, 'filter_fm_env3', 'FM<ENV3', -63, 63);
	setupKnob(101, 32, 'filter_fm_press', 'FM<PRESS', -63, 63);
	$scope.filter_fixmod = {pitchbend: false, vibrato: false};
	setupFields(28, 25, 'filter_fixmod', ['pitchbend', 'vibrato']);
	$scope.filter_keymod = {portamento: false, keyboard: false};
	setupFields(29, 26, 'filter_keymod', ['portamento', 'keyboard']);
	
	// Matrix
	$scope.matrix_parameters = [];
	for (var i = 0; i < 10; i++) {
		var oModel = {
			source: 0,
			dest: 0,
			value: 0,
			options: 0
		};
		$scope.matrix_parameters.push(oModel);
		setupMatrixKnob(i);
	}

	// ENV1
	setupKnob(50, 50, 'env1_delay', 'DELAY', 0, 63, 50);
	setupKnob(51, 51, 'env1_a', 'A', 0, 63, 50);
	setupKnob(52, 52, 'env1_d', 'D', 0, 63, 50);
	setupKnob(53, 53, 'env1_s', 'S', 0, 63, 50);
	setupKnob(54, 54, 'env1_r', 'R', 0, 63, 50);
	setupKnob(55, 55, 'env1_amp', 'AMP', 0, 63, 50);
	setupKnob(94, 56, 'env1_vel', 'VEL', -63, 63, 50);
	$scope.env1_trigger = {reset: false, multi: false, external: false};
	setupFields(49, 57, 'env1_trigger', ['reset', 'multi', 'external']);
	$scope.env1_lfotrigger = {gated: false, lfo: false};
	setupFields(56, 59, 'env1_lfotrigger', ['gated', 'lfo']);
	$scope.env1_mode = {dadr: false, free: false};
	setupFields(57, 58, 'env1_mode', ['dadr', 'free']);
	
	// ENV2
	setupKnob(59, 60, 'env2_delay', 'DELAY', 0, 63, 50);
	setupKnob(60, 61, 'env2_a', 'A', 0, 63, 50);
	setupKnob(61, 62, 'env2_d', 'D', 0, 63, 50);
	setupKnob(62, 63, 'env2_s', 'S', 0, 63, 50);
	setupKnob(63, 64, 'env2_r', 'R', 0, 63, 50);
	setupKnob(64, 65, 'env2_amp', 'AMP', 0, 63, 50);
	setupKnob(95, 66, 'env2_vel', 'VEL', -63, 63, 50);
	$scope.env2_trigger = {reset: false, multi: false, external: false};
	setupFields(65, 69, 'env2_trigger', ['reset', 'multi', 'external']);
	$scope.env2_lfotrigger = {gated: false, lfo: false};
	setupFields(65, 69, 'env2_lfotrigger', ['gated', 'lfo']);
	$scope.env2_mode = {dadr: false, free: false};
	setupFields(66, 68, 'env2_mode', ['dadr', 'free']);
	
	// ENV3
	setupKnob(68, 70, 'env3_delay', 'DELAY', 0, 63, 50);
	setupKnob(69, 71, 'env3_a', 'A', 0, 63, 50);
	setupKnob(70, 72, 'env3_d', 'D', 0, 63, 50);
	setupKnob(71, 73, 'env3_s', 'S', 0, 63, 50);
	setupKnob(72, 74, 'env3_r', 'R', 0, 63, 50);
	setupKnob(73, 75, 'env3_amp', 'AMP', 0, 63, 50);
	setupKnob(96, 76, 'env3_vel', 'VEL', -63, 63, 50);
	$scope.env3_trigger = {reset: false, multi: false, external: false};
	setupFields(67, 77, 'env3_trigger', ['reset', 'multi', 'external']);
	$scope.env3_lfotrigger = {gated: false, lfo: false};
	setupFields(74, 79, 'env3_lfotrigger', ['gated', 'lfo']);
	$scope.env3_mode = {dadr: false, free: false};
	setupFields(75, 78, 'env3_mode', ['dadr', 'free']);

	// RAMP1
	setupKnob(82, 40, 'ramp1_rate', 'RATE', 0, 63, 50);
	$scope.ramp1_mode = 0;
	$scope.$watch('ramp1_mode', function() {
		MIDIRequestService.sendParameter(41, $scope.ramp1_mode);
		PatchService.setValueAtIndex($scope.ramp1_mode, 83);
	});

	// RAMP2
	setupKnob(84, 42, 'ramp2_rate', 'RATE', 0, 63, 50);
	$scope.ramp2_mode = 0;
	$scope.$watch('ramp2_mode', function() {
		MIDIRequestService.sendParameter(43, $scope.ramp2_mode);
		PatchService.setValueAtIndex($scope.ramp2_mode, 85);
	});

	// TRACKING GENERATOR
	$scope.track_point_options = {
		floor: 0, 
		ceil: 63, 
		vertical: true,
		getPointerColor: function() { return '#245269'; },
		getSelectionBarColor: function() { return '#777777'; }
	};
	$scope.track_point_1 = 0;
	$scope.track_point_2 = 0;
	$scope.track_point_3 = 0;
	$scope.track_point_4 = 0;
	$scope.track_point_5 = 0;
	for (var i = 0; i < 5; i++) {
		$scope.$watch('track_point_'+i, function(newValue) {
			MIDIRequestService.sendParameter(34+i, newValue);
			PatchService.setValueAtIndex(newValue, 77+i);
		});
	}
	$scope.track_source = 0;
	$scope.$watch('track_source', function() {
		MIDIRequestService.sendParameter(33, $scope.track_source);
		PatchService.setValueAtIndex($scope.track_source, 76);
	});
	
	// PORTAMENTO
	setupKnob(32, 44, 'porta_rate', 'RATE', 0, 63);
	setupKnob(99, 45, 'porta_rate_vel', '<VEL', -63, 63);
	$scope.porta_legato = {legato: false};
	setupFields(34, 47, 'porta_legato', ['legato']);
	$scope.porta_lag_mode = 0;
	$scope.$watch('porta_lag_mode', function() {
		MIDIRequestService.sendParameter(46, $scope.porta_lag_mode);
		PatchService.setValueAtIndex($scope.porta_lag_mode, 33);
	});
	
	
	$rootScope.$on('patch-loaded', function() {
		$log.debug('patch-loaded');
		// rafraichir les parametres
		// desactiver la sortie car le changement des valeurs provoque des sendParameters inutiles (à voir si 
		// c'est la meilleurs solution)
		MIDIService.setOutputEnabled(false);
		
		$scope.$apply(function() {
			for (var i = 0; i < parameters.length; i++) {
				var oParam = parameters[i];
				$scope[oParam.knobValue] = PatchService.getValueAtIndex(oParam.paramIndex);
			}
			for (var i = 0; i < signedParameters.length; i++) {
				var oParam = signedParameters[i];
				var oValue = PatchService.getValueAtIndex(oParam.paramIndex);
				$scope[oParam.knobValue] = oValue;
				/*
				if (oParam.paramIndex === 19) {
					console.log('detune: '+$scope[oParam.knobValue]);
				}*/
			}
			for (var i = 0; i < bitFieldParameters.length; i++) {
				var oParam = bitFieldParameters[i];
				var oValue = PatchService.getValueAtIndex(oParam.paramIndex);
				for (var p = 0; p < oParam.properties.length; p++) {
					var oBit = (oValue & (1 << p)) >> p;
					$scope[oParam.varName][oParam.properties[p]] = oBit === 1;
				}
			}
			for (var i = 0; i < selectParameters.length; i++) {
				var oParam = selectParameters[i];
				var oValueIndex = PatchService.getValueAtIndex(oParam.paramIndex);
				var oValue = oParam.values[oValueIndex];
				$scope[oParam.name] = oValue;
			}
			$scope.lfo1_trigger = PatchService.getValueAtIndex(36);
			$scope.lfo1_wave = PatchService.getValueAtIndex(38);
			$scope.lfo1_sample_source = PatchService.getValueAtIndex(40);
			
			$scope.lfo2_trigger = PatchService.getValueAtIndex(43);
			$scope.lfo2_wave = PatchService.getValueAtIndex(45);
			$scope.lfo2_sample_source = PatchService.getValueAtIndex(47);

			// mod matrix
			for (var i = 0; i < 10; i++) {
				$scope.matrix_parameters[i].source = PatchService.getValueAtIndex(104+i*3);
				$scope.matrix_parameters[i].value = PatchService.getValueAtIndex(105+i*3);
				$scope.matrix_parameters[i].dest = PatchService.getValueAtIndex(106+i*3);
			}
			
			// tracking generator
			for (var i = 0; i < 5; i++) {
				$scope['track_point_'+i] = PatchService.getValueAtIndex(77+i);
			}
			$scope.track_source = PatchService.getValueAtIndex(76);
			
			$scope.porta_lag_mode = PatchService.getValueAtIndex(33);
			$scope.ramp1_mode = PatchService.getValueAtIndex(83);
			$scope.ramp2_mode = PatchService.getValueAtIndex(85);
			
		});
		
		MIDIService.setOutputEnabled(true);
	});
}])

/**
 * La directive attribut pour le select.
 */
.directive('convertToNumber', function () {
	return {
		require: 'ngModel',
		link: function (scope, element, attrs, ngModel) {
			ngModel.$parsers.push(function (val) {
				return parseInt(val, 10);
			});
			ngModel.$formatters.push(function (val) {
				return '' + val;
			});
		}
	};
});
