/*global console,_ */
'use strict';

var oMIDIModule = angular.module('MIDI');

oMIDIModule.constant('IDW', 0x10) // Oberheim
	.constant('IDE', 6) // Matrix-1000
;

oMIDIModule.service('MIDIRequestService', ['$log', '$timeout', 'MIDIService', 'IDW', 'IDE', 
	
function($log, $timeout, MIDIService, IDW, IDE) {
	
	var oService = this;				
	
	this.sendParameter = _.throttle(function(aParameter, aParameterValue) {
		// f0 10 06 06 15 26 f7
		if (aParameter === 12) {
			// detune case : in: -31..31
			if (aParameterValue < 0)
			{
				aParameterValue = 128 + aParameterValue;
			}
		}
		else {
			if (aParameterValue < 0) {
				aParameterValue += 128;
			}
		}
		var oSysex = [0xf0, IDW, IDE, 0x06, aParameter, aParameterValue & 0x7F, 0xf7];
		MIDIService.sendData(oSysex);
	}, MIDIService.getThrottleDelay());
	
	
	/**
	 * Set a parameter of the modulation matrix.
	 * @param {integer} aPath 0..9
	 * @param {integer} aSource
	 * @param {integer} aDest
	 * @param {integer} aValue
	 */
	this.sendMatrixParameter = _.throttle(function(aPath, aSource, aDest, aValue) {
		if (aValue < 0) {
			aValue += 128;
		}
		var oSysex = [0xf0, IDW, IDE, 0x0B, aPath, aSource, aValue, aDest, 0xf7];
		MIDIService.sendData(oSysex);
	}, MIDIService.getThrottleDelay());
	
	this.requestEditBuffer = function(aReceiveHandler) {
		MIDIService.setReceiveHandler(aReceiveHandler);
		return oService.requestData(4, 0);
	};
	
	this.requestData = function(aType, aNumber, aRecursiveCall) {
		var oOutput = MIDIService.getOutputId();
		if (!oOutput) {
			$log.error("No MIDI output selected");
			if (!aRecursiveCall) {
				// At navigator startup, MIDI port are not all available, wait a little and retry
				$log.debug('Wait a little bit and retry');
				$timeout(function() {
					oService.requestData(aType, aNumber, true);
				}, 500);
			}
			return;
		}
		var oSysexBytes = [0xF0, IDW, IDE, 0x04, aType, aNumber, 0xF7];
		MIDIService.sendData(oSysexBytes);
	};
	

}]);

