/*global console */
'use strict';

var oModule = angular.module('Patch', []);

oModule.service('PatchService', ['$rootScope', function($rootScope) {

	var SYSEX_CMD_OFFSET = 3;
	var SYSEX_PATCH_OFFSET = 5;
	var PATCH_PARAMETERS_COUNT = 134;
	var PATCH_BYTES_COUNT = 2*PATCH_PARAMETERS_COUNT;
	var BYTES_COUNT = PATCH_BYTES_COUNT + SYSEX_PATCH_OFFSET + 1 + 1;
	var CHECKSUM_BYTE_OFFSET = BYTES_COUNT - 2;
	var MATRIX_INDEX_OFFSET = 104;
	var DETUNE_OFFSET = 19;
	var NAME_LENGTH = 8;
	
	var oService = this;	
	/** le sysex du patch */
	var data;
	
	
	
	this.setData = function(aData) {
		//console.log('data: '+aData);
		oService.data = aData;
		$rootScope.$broadcast('patch-loaded');
	};
	
	this.setDataFromString = function(aDataString) {
		var oDataSplit = aDataString.split(',');
		var oTab = [];
		for (var i = 0; i < oDataSplit.length; i++) {
			oTab.push(parseInt(oDataSplit[i]));
		}
		oService.setData(oTab);
	};
	
	
	/**
	 * Returns sysex array.
	 * @returns {Array}
	 */
	this.getData = function() {
		oService.data[CHECKSUM_BYTE_OFFSET] = oService.getChecksum();
		return oService.data;
	};
	
	this.initPatch = function(aPatchSysex) {
		if (aPatchSysex.length === PATCH_PARAMETERS_COUNT) {
			return 'This file comes from OSX editor : not yet supported';
		}
		if (aPatchSysex[0] !== 0xF0) {
			return 'Not a sysex';
		}
		if (aPatchSysex.length !== BYTES_COUNT) {
			return 'Not a valid Matrix patch sysex';
		}
		oService.setData(aPatchSysex);
		// recalculer le CS
		oService.data[CHECKSUM_BYTE_OFFSET] = oService.getChecksum();
		if (localStorage) {
			localStorage.setItem('com.dqueffeulou.ObieEditor.patch', aPatchSysex);
		}
		return undefined;
	};
	
	this.initPatchFromDataView = function(aDataView) {
		if (aDataView.byteLength === PATCH_PARAMETERS_COUNT) {
			return 'This file comes from OSX editor : not yet supported';
		}
		if (aDataView.byteLength !== BYTES_COUNT) {
			return 'Not a valid Matrix patch sysex';
		}
		var oByteArray = [];
		for (var i = 0; i < aDataView.byteLength; i++) {
			oByteArray[i] = aDataView.getInt8(i) & 0xFF;
		}
		return this.initPatch(oByteArray);
	};
	
	
	this.getDataToSendToEditBuffer = function() {
		var oData = this.getData();
		oData[SYSEX_CMD_OFFSET] = 0x0D;
		return oData;
	};
	
	
	/**
	 * 
	 * @param aIndex l'index d'un parametre, indiqué dans la doc 0..133
	 * @return 
	 */
	function getEvenByteAtIndex(aIndex)
	{
		if (!oService.data) { return 0; }
		return oService.data[SYSEX_PATCH_OFFSET + aIndex*2];
	}

	function getOddByteAtIndex(aIndex)
	{
		if (!oService.data) { return 0; }
		return oService.data[SYSEX_PATCH_OFFSET + aIndex*2 + 1];
	}
	
	function setEvenByteAtIndex(aByte, aIndex)
	{
		if (!oService.data) { return 0; }
		oService.data[SYSEX_PATCH_OFFSET + aIndex*2] = aByte;
	}
	
	function setOddByteAtIndex(aByte, aIndex)
	{
		if (!oService.data) { return 0; }
		oService.data[SYSEX_PATCH_OFFSET + aIndex*2 + 1] = aByte;
	}
	
	/**
	 * Return the value of the parameter.
	 * @param aIndex l'index d'un parametre, indiqué dans la doc 0..133
	 * @return -63..127
	 */
	this.getValueAtIndex = function(aIndex)
	{
		var oVal = (getEvenByteAtIndex(aIndex) + (getOddByteAtIndex(aIndex) << 4)) & 0x7F;
		if (aIndex === 26) {
			// VCF 7 bits not signed => no change
		}
		else if (aIndex === 19) {
			// detune out:-31..31
			oVal = oVal & 0x3F;
			// 32 (-0) is not supposed to exist but...
			if (oVal === 32) {
				oVal = 0;
			}
			else if (oVal > 31) {
				oVal = oVal - 64;
			}
		}
		else if (oVal > 63) {
			// other parameters > 63 are signed parameters on 7 bits
			// out: -63..63
			oVal -= 128;
		}
		return oVal;
	};
	
	/**
	 * 
	 * @param aValue -63..127
	 * @param aIndex l'index d'un parametre, indiqué dans la doc 0..133
	 */
	this.setValueAtIndex = function(aValue, aIndex)
	{
		if (aValue < 0) {
			aValue += 128;
		}
		if (aIndex === 19) {
			// detune
			aValue = aValue & 0x3F;
		}
		setEvenByteAtIndex((aValue & 0x0F), aIndex);
		setOddByteAtIndex(((aValue >> 4) & 0x0F), aIndex);
		oService.setDirty(true);
	};
	
	
	
	
	this.setBitValueAtIndex = function(aBitOffset, aValue, aIndex)
	{
		var oValueToSet = aValue ? 1 : 0;
		var oVal = getEvenByteAtIndex(aIndex);
		oVal &= ~(1 << aBitOffset);
		oVal |= (oValueToSet << aBitOffset);
		setValueAtIndex(oVal, aIndex);
		setEvenByteAtIndex(oVal, aIndex);
		setDirty(true);
	};
	
	this.getBitValueAtIndex = function(aBitOffset, aIndex)
	{
		var oVal = getEvenByteAtIndex(aIndex);
		oVal = (oVal >> aBitOffset) & 1;
		return oVal === 1;
	};
	
	/**
	 * Calcule le checksum.
	 */
	this.getChecksum = function()
	{
		// calculer le checksum
		var oCS = 0;
		for (var i = 0; i < PATCH_PARAMETERS_COUNT; i++)
		{
			oCS += oService.getValueAtIndex(i);
		}
		return oCS & 0x7F;
	};
	
	/**
	 * Indique si le patch a été modifié depuis sa dernière sauvegarde
	 * ou si il n'a jamais été sauvegardé.
	 * @return the dirty
	 */
	this.isDirty = function()
	{
		return dirty;
	};

	/**
	 * @param dirty the dirty to set
	 */
	this.setDirty = function(dirty)
	{
		this.dirty = dirty;
/*		if (patchDirtyListener != null)
		{
			patchDirtyListener.dirtyChange(dirty);
		}*/
	};

	
/*		
	public void setPatchDirtyListener(final PatchDirtyListener aPatchDirtyListener)
	{
		patchDirtyListener = aPatchDirtyListener;
	}
*/	

}]);

oModule.run(['$rootScope', 'PatchService', 'MIDIRequestService', function($rootScope, PatchService, MIDIRequestService) {

	$rootScope.$on('midi-ready', function() {
		MIDIRequestService.requestEditBuffer(function (aData) {
			PatchService.initPatch(aData);
		});
	});
/*		
	if (localStorage) {
		// recharger le patch sauvegardé
		var oData = localStorage.getItem('com.dqueffeulou.ObieEditor.patch');
		if (oData) {
			// il est retourné sous forme de chaine de caractères
			PatchService.setDataFromString(oData);
		}
	}
*/

}]);

